@@ -2,6 +2,7 @@ from math import log2, sqrt
 import torch
 from torch import nn, einsum
 import torch.nn.functional as F
+import numpy as np
 
 from axial_positional_embedding import AxialPositionalEmbedding
 from einops import rearrange
@@ -18,10 +19,11 @@ def exists(val):
 def default(val, d):
     return val if exists(val) else d
 
-def always(val):
-    def inner(*args, **kwargs):
-        return val
-    return inner
+class always():
+    def __init__(self, val):
+        self.val = val
+    def __call__(self, x, *args, **kwargs):
+        return self.val
 
 def is_empty(t):
     return t.nelement() == 0
@@ -321,7 +323,9 @@ class DALLE(nn.Module):
         sparse_attn = False,
         attn_types = None,
         loss_img_weight = 7,
-        stable = False
+        stable = False,
+        shift_tokens = False,
+        rotary_emb = False
     ):
         super().__init__()
         assert isinstance(vae, (DiscreteVAE, OpenAIDiscreteVAE, VQGanVAE)), 'vae must be an instance of DiscreteVAE'
@@ -336,8 +340,8 @@ class DALLE(nn.Module):
         self.text_emb = nn.Embedding(num_text_tokens, dim)
         self.image_emb = nn.Embedding(num_image_tokens, dim)
 
-        self.text_pos_emb = nn.Embedding(text_seq_len + 1, dim) # +1 for <bos>
-        self.image_pos_emb = AxialPositionalEmbedding(dim, axial_shape = (image_fmap_size, image_fmap_size))
+        self.text_pos_emb = nn.Embedding(text_seq_len + 1, dim) if not rotary_emb else always(0) # +1 for <bos>
+        self.image_pos_emb = AxialPositionalEmbedding(dim, axial_shape = (image_fmap_size, image_fmap_size)) if not rotary_emb else always(0)
 
         self.num_text_tokens = num_text_tokens # for offsetting logits index and calculating cross entropy loss
         self.num_image_tokens = num_image_tokens
@@ -366,7 +370,9 @@ class DALLE(nn.Module):
             attn_types = attn_types,
             image_fmap_size = image_fmap_size,
             sparse_attn = sparse_attn,
-            stable = stable
+            stable = stable,
+            shift_tokens = shift_tokens,
+            rotary_emb = rotary_emb
         )
 
         self.stable = stable
@@ -393,6 +399,55 @@ class DALLE(nn.Module):
         self.register_buffer('logits_mask', logits_mask, persistent=False)
         self.loss_img_weight = loss_img_weight
 
+
+    @torch.no_grad()
+    @eval_decorator
+    def generate_texts(
+        self,
+        tokenizer,
+        text = None,
+        *,
+        filter_thres = 0.5,
+        temperature = 1.
+    ):
+        text_seq_len = self.text_seq_len
+        if text is None or text == "":
+            text_tokens = torch.tensor([[0]]).cuda()
+        else:
+            text_tokens = torch.tensor(tokenizer.tokenizer.encode(text)).cuda().unsqueeze(0)
+   
+        for _ in range(text_tokens.shape[1], text_seq_len):
+            device = text_tokens.device
+
+            tokens = self.text_emb(text_tokens)
+            tokens += self.text_pos_emb(torch.arange(text_tokens.shape[1], device = device))
+
+            seq_len = tokens.shape[1]
+
+            output_transf = self.transformer(tokens)
+
+            if self.stable:
+                output_transf = self.norm_by_max(output_transf)
+
+            logits = self.to_logits(output_transf)
+
+            # mask logits to make sure text predicts text (except last token), and image predicts image
+
+            logits_mask = self.logits_mask[:, :seq_len]
+            max_neg_value = -torch.finfo(logits.dtype).max
+            logits.masked_fill_(logits_mask, max_neg_value)
+            logits = logits[:, -1, :]
+
+            filtered_logits = top_k(logits, thres = filter_thres)
+            probs = F.softmax(filtered_logits / temperature, dim = -1)
+            sample = torch.multinomial(probs, 1)
+ 
+            text_tokens = torch.cat((text_tokens, sample), dim=-1)
+    
+        padding_tokens = set(np.arange(self.text_seq_len) + (self.num_text_tokens - self.text_seq_len))
+        texts = [tokenizer.tokenizer.decode(text_token, pad_tokens=padding_tokens) for text_token in text_tokens]
+        return text_tokens, texts
+
     @torch.no_grad()
     @eval_decorator
     def generate_images(
